==> gpxview_0.8.14.tar.gz: gpxview-0.8.14/src/custom_type_renderer.c <==
/*
 * Copyright (C) 2008 Till Harbaum <till@harbaum.org>.
 *
 * This file is part of GPXView.
 *
 * GPXView is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GPXView is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GPXView.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gpxview.h"
#include "custom_type_renderer.h"

#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR >= 5)
#define CUSTOM_ICON_TYPE  ICON_CACHE_TYPE_1_5X
#else
#define CUSTOM_ICON_TYPE  ICON_CACHE_TYPE
#endif

/* http://scentric.net/tutorial/sec-custom-cell-renderers.html */
/* https://stage.maemo.org/svn/maemo/projects/haf/trunk/gtk+/gtk/gtkcellrendererpixbuf.c */
/* https://stage.maemo.org/svn/maemo/projects/haf/trunk/gtk+/gtk/gtkcellrenderertext.c */

/* This is based mainly on GtkCellRendererProgress
 *  in GAIM, written and (c) 2002 by Sean Egan
 *  (Licensed under the GPL), which in turn is
 *  based on Gtk's GtkCellRenderer[Text|Toggle|Pixbuf]
 *  implementation by Jonathan Blandford */

/* Some boring function declarations: GObject type system stuff */

static void     custom_cell_renderer_type_init       (CustomCellRendererType      *celltype);
static void     custom_cell_renderer_type_class_init (CustomCellRendererTypeClass *klass);
static void     custom_cell_renderer_type_get_property  (GObject                    *object,
                                                             guint                       param_id,
                                                             GValue                     *value,
                                                             GParamSpec                 *pspec);
static void     custom_cell_renderer_type_set_property  (GObject                    *object,
                                                             guint                       param_id,
                                                             const GValue               *value,
                                                             GParamSpec                 *pspec);
static void     custom_cell_renderer_type_finalize (GObject *gobject);


/* These functions are the heart of our custom cell renderer: */

static void     custom_cell_renderer_type_get_size   (GtkCellRenderer            *cell,
                                                          GtkWidget                  *widget,
                                                          GdkRectangle               *cell_area,
                                                          gint                       *x_offset,
                                                          gint                       *y_offset,
                                                          gint                       *width,
                                                          gint                       *height);

static void     custom_cell_renderer_type_render     (GtkCellRenderer            *cell,
                                                          GdkWindow                  *window,
                                                          GtkWidget                  *widget,
                                                          GdkRectangle               *background_area,
                                                          GdkRectangle               *cell_area,
                                                          GdkRectangle               *expose_area,
                                                          guint                       flags);


enum { PROP_TYPE = 1, };

static gpointer parent_class;

GType custom_cell_renderer_type_get_type (void) {
  static GType cell_type_type = 0;

  if (cell_type_type == 0)
  {
    static const GTypeInfo cell_type_info =
    {
      sizeof (CustomCellRendererTypeClass),
      NULL,                                                     /* base_init */
      NULL,                                                     /* base_finalize */
      (GClassInitFunc) custom_cell_renderer_type_class_init,
      NULL,                                                     /* class_finalize */
      NULL,                                                     /* class_data */
      sizeof (CustomCellRendererType),
      0,                                                        /* n_preallocs */
      (GInstanceInitFunc) custom_cell_renderer_type_init,
    };

    /* Derive from GtkCellRenderer */
    cell_type_type = g_type_register_static (GTK_TYPE_CELL_RENDERER,
					     "CustomCellRendererType",
					     &cell_type_info,
					     0);
  }
  return cell_type_type;
}


/***************************************************************************
 *
 *  custom_cell_renderer_type_init: set some default properties of the
 *                                      parent (GtkCellRenderer).
 *
 ***************************************************************************/

static void
custom_cell_renderer_type_init (CustomCellRendererType *cellrenderertype)
{
  GTK_CELL_RENDERER(cellrenderertype)->mode = GTK_CELL_RENDERER_MODE_INERT;
  GTK_CELL_RENDERER(cellrenderertype)->xpad = 0;
  GTK_CELL_RENDERER(cellrenderertype)->ypad = 0;
}


/***************************************************************************
 *
 *  custom_cell_renderer_type_class_init:
 *
 *  set up our own get_property and set_property functions, and
 *  override the parent's functions that we need to implement.
 *  And make our new "size" property known to the type system.
 *  If you want cells that can be activated on their own (ie. not
 *  just the whole row selected) or cells that are editable, you
 *  will need to override 'activate' and 'start_editing' as well.
 *
 ***************************************************************************/

static void
custom_cell_renderer_type_class_init (CustomCellRendererTypeClass *klass) {
  GtkCellRendererClass *cell_class   = GTK_CELL_RENDERER_CLASS(klass);
  GObjectClass         *object_class = G_OBJECT_CLASS(klass);

  parent_class           = g_type_class_peek_parent (klass);
  object_class->finalize = custom_cell_renderer_type_finalize;

  /* Hook up functions to set and get our
   *   custom cell renderer properties */
  object_class->get_property = custom_cell_renderer_type_get_property;
  object_class->set_property = custom_cell_renderer_type_set_property;

  /* Override the two crucial functions that are the heart
   *   of a cell renderer in the parent class */
  cell_class->get_size = custom_cell_renderer_type_get_size;
  cell_class->render   = custom_cell_renderer_type_render;

  /* Install our very own properties */
  g_object_class_install_property (object_class, PROP_TYPE,
    g_param_spec_int ("type", "Type", "Container type", -256, 
		      (CACHE_TYPE_MAX<<8)+255, 0, G_PARAM_READWRITE));
}


/***************************************************************************
 *
 *  custom_cell_renderer_type_finalize: free any resources here
 *
 ***************************************************************************/

static void custom_cell_renderer_type_finalize (GObject *object) {
  (* G_OBJECT_CLASS (parent_class)->finalize) (object);
}


/***************************************************************************
 *
 *  custom_cell_renderer_type_get_property: as it says
 *
 ***************************************************************************/

static void
custom_cell_renderer_type_get_property (GObject    *object,
					guint       param_id,
					GValue     *value,
					GParamSpec *psec) {
  CustomCellRendererType  *celltype = CUSTOM_CELL_RENDERER_TYPE(object);

  switch (param_id) {
    case PROP_TYPE:
      g_value_set_int(value, celltype->type);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, psec);
      break;
  }
}


/***************************************************************************
 *
 *  custom_cell_renderer_type_set_property: as it says
 *
 ***************************************************************************/

static void
custom_cell_renderer_type_set_property (GObject      *object,
                                            guint         param_id,
                                            const GValue *value,
                                            GParamSpec   *pspec) {
  CustomCellRendererType *celltype = CUSTOM_CELL_RENDERER_TYPE (object);

  switch (param_id) {
    case PROP_TYPE:
      celltype->type = g_value_get_int(value);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
      break;
  }
}

/***************************************************************************
 *
 *  custom_cell_renderer_type_new: return a new cell renderer instance
 *
 ***************************************************************************/

GtkCellRenderer *custom_cell_renderer_type_new (void) {
  return g_object_new(CUSTOM_TYPE_CELL_RENDERER_TYPE, NULL);
}

static void
custom_cell_renderer_type_get_size (GtkCellRenderer *cell,
   GtkWidget *widget, GdkRectangle *cell_area, gint *x_offset, 
   gint *y_offset, gint *width, gint *height) {
  CustomCellRendererType *celltype = CUSTOM_CELL_RENDERER_TYPE (cell);
  GdkPixbuf *pixbuf = icon_get(CUSTOM_ICON_TYPE, celltype->type>>8);

  if(!pixbuf) return;

  gint calc_width  = (gint) gdk_pixbuf_get_width(pixbuf);
  gint calc_height  = (gint) gdk_pixbuf_get_height(pixbuf);

  if (x_offset) *x_offset = 0;
  if (y_offset) *y_offset = 0;

  if (cell_area) {
    if (x_offset) 
      *x_offset = (cell_area->width - calc_width)/2;

    if (y_offset)
      *y_offset = (cell_area->height - calc_height)/2;
  }

  if (width)  *width = calc_width;
  if (height) *height = calc_height;
}


/***************************************************************************
 *
 *  custom_cell_renderer_type_render: crucial - do the rendering.
 *
 ***************************************************************************/

static void
custom_cell_renderer_type_render (GtkCellRenderer *cell,
                                      GdkWindow       *window,
                                      GtkWidget       *widget,
                                      GdkRectangle    *background_area,
                                      GdkRectangle    *cell_area,
                                      GdkRectangle    *expose_area,
                                      guint            flags)
{
  CustomCellRendererType *celltype = CUSTOM_CELL_RENDERER_TYPE (cell);
  GdkPixbuf *pixbuf = icon_get(CUSTOM_ICON_TYPE, celltype->type>>8);
  GdkRectangle all_rect;
  GdkRectangle draw_rect;

  if(!pixbuf) return;

  gint pix_width  = gdk_pixbuf_get_width(pixbuf);

  custom_cell_renderer_type_get_size (cell, widget, cell_area,
				     &all_rect.x, &all_rect.y,
				     &all_rect.width, &all_rect.height);

  all_rect.x += cell_area->x;
  all_rect.y += cell_area->y;
  
  if (!gdk_rectangle_intersect (cell_area, &all_rect, &draw_rect) ||
      !gdk_rectangle_intersect (expose_area, &draw_rect, &draw_rect))
    return;

  /* draw the bitmap */
  cairo_t *cr = gdk_cairo_create(window);

  gdk_cairo_set_source_pixbuf(cr, pixbuf, 
	      all_rect.x + (all_rect.width - pix_width)/2, all_rect.y);
  gdk_cairo_rectangle(cr, &draw_rect);
  cairo_fill(cr);

  /* just a note, nothing else */
  GdkPixbuf *addon = NULL;
  if((celltype->type & 7) == 4)      // only note is set
    addon = icon_get(CUSTOM_ICON_TYPE, 13);
  if((celltype->type & 7) == 5)      // only note and override are set
    addon = icon_get(CUSTOM_ICON_TYPE, 11);
  if(celltype->type & 2)             // solved is set
    addon = icon_get(CUSTOM_ICON_TYPE, 12);
  
  /* if override (and not found) is set */
  if(addon) {
    gdk_cairo_set_source_pixbuf(cr, addon, 
	all_rect.x + (all_rect.width - pix_width)/2, all_rect.y);
    gdk_cairo_rectangle(cr, &draw_rect);
    cairo_fill(cr);
  }

  cairo_destroy(cr);
}

==> gpxview_0.9.0.tar.gz: gpxview-0.9.0/src/custom_type_renderer.c <==
/*
 * Copyright (C) 2008 Till Harbaum <till@harbaum.org>.
 *
 * This file is part of GPXView.
 *
 * GPXView is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * GPXView is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GPXView.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gpxview.h"
#include "custom_type_renderer.h"

#if defined(USE_MAEMO) && (MAEMO_VERSION_MAJOR >= 5)
#define CUSTOM_ICON_TYPE  ICON_CACHE_TYPE_1_5X
#else
#define CUSTOM_ICON_TYPE  ICON_CACHE_TYPE
#endif

/* http://scentric.net/tutorial/sec-custom-cell-renderers.html */
/* https://stage.maemo.org/svn/maemo/projects/haf/trunk/gtk+/gtk/gtkcellrendererpixbuf.c */
/* https://stage.maemo.org/svn/maemo/projects/haf/trunk/gtk+/gtk/gtkcellrenderertext.c */

/* This is based mainly on GtkCellRendererProgress
 *  in GAIM, written and (c) 2002 by Sean Egan
 *  (Licensed under the GPL), which in turn is
 *  based on Gtk's GtkCellRenderer[Text|Toggle|Pixbuf]
 *  implementation by Jonathan Blandford */

/* Some boring function declarations: GObject type system stuff */

static void     custom_cell_renderer_type_init       (CustomCellRendererType      *celltype);
static void     custom_cell_renderer_type_class_init (CustomCellRendererTypeClass *klass);
static void     custom_cell_renderer_type_get_property  (GObject                    *object,
                                                             guint                       param_id,
                                                             GValue                     *value,
                                                             GParamSpec                 *pspec);
static void     custom_cell_renderer_type_set_property  (GObject                    *object,
                                                             guint                       param_id,
                                                             const GValue               *value,
                                                             GParamSpec                 *pspec);
static void     custom_cell_renderer_type_finalize (GObject *gobject);


/* These functions are the heart of our custom cell renderer: */

static void     custom_cell_renderer_type_get_size   (GtkCellRenderer            *cell,
                                                          GtkWidget                  *widget,
                                                          GdkRectangle               *cell_area,
                                                          gint                       *x_offset,
                                                          gint                       *y_offset,
                                                          gint                       *width,
                                                          gint                       *height);

static void     custom_cell_renderer_type_render     (GtkCellRenderer            *cell,
                                                          GdkWindow                  *window,
                                                          GtkWidget                  *widget,
                                                          GdkRectangle               *background_area,
                                                          GdkRectangle               *cell_area,
                                                          GdkRectangle               *expose_area,
                                                          guint                       flags);


enum { PROP_TYPE = 1, };

static gpointer parent_class;

GType custom_cell_renderer_type_get_type (void) {
  static GType cell_type_type = 0;

  if (cell_type_type == 0)
  {
    static const GTypeInfo cell_type_info =
    {
      sizeof (CustomCellRendererTypeClass),
      NULL,                                                     /* base_init */
      NULL,                                                     /* base_finalize */
      (GClassInitFunc) custom_cell_renderer_type_class_init,
      NULL,                                                     /* class_finalize */
      NULL,                                                     /* class_data */
      sizeof (CustomCellRendererType),
      0,                                                        /* n_preallocs */
      (GInstanceInitFunc) custom_cell_renderer_type_init,
    };

    /* Derive from GtkCellRenderer */
    cell_type_type = g_type_register_static (GTK_TYPE_CELL_RENDERER,
					     "CustomCellRendererType",
					     &cell_type_info,
					     0);
  }
  return cell_type_type;
}


/***************************************************************************
 *
 *  custom_cell_renderer_type_init: set some default properties of the
 *                                      parent (GtkCellRenderer).
 *
 ***************************************************************************/

static void
custom_cell_renderer_type_init (CustomCellRendererType *cellrenderertype)
{
  GTK_CELL_RENDERER(cellrenderertype)->mode = GTK_CELL_RENDERER_MODE_INERT;
  GTK_CELL_RENDERER(cellrenderertype)->xpad = 0;
  GTK_CELL_RENDERER(cellrenderertype)->ypad = 0;
}


/***************************************************************************
 *
 *  custom_cell_renderer_type_class_init:
 *
 *  set up our own get_property and set_property functions, and
 *  override the parent's functions that we need to implement.
 *  And make our new "size" property known to the type system.
 *  If you want cells that can be activated on their own (ie. not
 *  just the whole row selected) or cells that are editable, you
 *  will need to override 'activate' and 'start_editing' as well.
 *
 ***************************************************************************/

static void
custom_cell_renderer_type_class_init (CustomCellRendererTypeClass *klass) {
  GtkCellRendererClass *cell_class   = GTK_CELL_RENDERER_CLASS(klass);
  GObjectClass         *object_class = G_OBJECT_CLASS(klass);

  parent_class           = g_type_class_peek_parent (klass);
  object_class->finalize = custom_cell_renderer_type_finalize;

  /* Hook up functions to set and get our
   *   custom cell renderer properties */
  object_class->get_property = custom_cell_renderer_type_get_property;
  object_class->set_property = custom_cell_renderer_type_set_property;

  /* Override the two crucial functions that are the heart
   *   of a cell renderer in the parent class */
  cell_class->get_size = custom_cell_renderer_type_get_size;
  cell_class->render   = custom_cell_renderer_type_render;

  /* Install our very own properties */
  g_object_class_install_property (object_class, PROP_TYPE,
    g_param_spec_int ("type", "Type", "Container type", -256, 
		      (CACHE_TYPE_MAX<<8)+255, 0, G_PARAM_READWRITE));
}


/***************************************************************************
 *
 *  custom_cell_renderer_type_finalize: free any resources here
 *
 ***************************************************************************/

static void custom_cell_renderer_type_finalize (GObject *object) {
  (* G_OBJECT_CLASS (parent_class)->finalize) (object);
}


/***************************************************************************
 *
 *  custom_cell_renderer_type_get_property: as it says
 *
 ***************************************************************************/

static void
custom_cell_renderer_type_get_property (GObject    *object,
					guint       param_id,
					GValue     *value,
					GParamSpec *psec) {
  CustomCellRendererType  *celltype = CUSTOM_CELL_RENDERER_TYPE(object);

  switch (param_id) {
    case PROP_TYPE:
      g_value_set_int(value, celltype->type);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, psec);
      break;
  }
}


/***************************************************************************
 *
 *  custom_cell_renderer_type_set_property: as it says
 *
 ***************************************************************************/

static void
custom_cell_renderer_type_set_property (GObject      *object,
                                            guint         param_id,
                                            const GValue *value,
                                            GParamSpec   *pspec) {
  CustomCellRendererType *celltype = CUSTOM_CELL_RENDERER_TYPE (object);

  switch (param_id) {
    case PROP_TYPE:
      celltype->type = g_value_get_int(value);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
      break;
  }
}

/***************************************************************************
 *
 *  custom_cell_renderer_type_new: return a new cell renderer instance
 *
 ***************************************************************************/

GtkCellRenderer *custom_cell_renderer_type_new (void) {
  return g_object_new(CUSTOM_TYPE_CELL_RENDERER_TYPE, NULL);
}

static void
custom_cell_renderer_type_get_size (GtkCellRenderer *cell,
   GtkWidget *widget, GdkRectangle *cell_area, gint *x_offset, 
   gint *y_offset, gint *width, gint *height) {
  CustomCellRendererType *celltype = CUSTOM_CELL_RENDERER_TYPE (cell);
  GdkPixbuf *pixbuf = icon_get(CUSTOM_ICON_TYPE, celltype->type>>8);

  if(!pixbuf) return;

  gint calc_width  = (gint) gdk_pixbuf_get_width(pixbuf);
  gint calc_height  = (gint) gdk_pixbuf_get_height(pixbuf);

  if (x_offset) *x_offset = 0;
  if (y_offset) *y_offset = 0;

  if (cell_area) {
    if (x_offset) 
      *x_offset = (cell_area->width - calc_width)/2;

    if (y_offset)
      *y_offset = (cell_area->height - calc_height)/2;
  }

  if (width)  *width = calc_width;
  if (height) *height = calc_height;
}


/***************************************************************************
 *
 *  custom_cell_renderer_type_render: crucial - do the rendering.
 *
 ***************************************************************************/

static void
custom_cell_renderer_type_render (GtkCellRenderer *cell,
                                      GdkWindow       *window,
                                      GtkWidget       *widget,
                                      GdkRectangle    *background_area,
                                      GdkRectangle    *cell_area,
                                      GdkRectangle    *expose_area,
                                      guint            flags)
{
  CustomCellRendererType *celltype = CUSTOM_CELL_RENDERER_TYPE (cell);
  GdkPixbuf *pixbuf = icon_get(CUSTOM_ICON_TYPE, celltype->type>>8);
  GdkRectangle all_rect;
  GdkRectangle draw_rect;

  if(!pixbuf) return;

  gint pix_width  = gdk_pixbuf_get_width(pixbuf);

  custom_cell_renderer_type_get_size (cell, widget, cell_area,
				     &all_rect.x, &all_rect.y,
				     &all_rect.width, &all_rect.height);

  all_rect.x += cell_area->x;
  all_rect.y += cell_area->y;
  
  if (!gdk_rectangle_intersect (cell_area, &all_rect, &draw_rect) ||
      !gdk_rectangle_intersect (expose_area, &draw_rect, &draw_rect))
    return;

  /* draw the bitmap */
  cairo_t *cr = gdk_cairo_create(window);

  gdk_cairo_set_source_pixbuf(cr, pixbuf, 
	      all_rect.x + (all_rect.width - pix_width)/2, all_rect.y);
  gdk_cairo_rectangle(cr, &draw_rect);
  cairo_fill(cr);

  /* add "special" addon. the order of these test results in */
  /* a priority */
  GdkPixbuf *addon = NULL;
  if(celltype->type & 8)             // mine is set
    addon = icon_get(CUSTOM_ICON_TYPE, 14);
  else if(celltype->type & 2)        // solved is set
    addon = icon_get(CUSTOM_ICON_TYPE, 12);
  else if(celltype->type & 1)        // override is set
    addon = icon_get(CUSTOM_ICON_TYPE, 11);
  else if(celltype->type & 4)        // note is set
    addon = icon_get(CUSTOM_ICON_TYPE, 13);
  
  /* if override (and not found) is set */
  if(addon) {
    gdk_cairo_set_source_pixbuf(cr, addon, 
	all_rect.x + (all_rect.width - pix_width)/2, all_rect.y);
    gdk_cairo_rectangle(cr, &draw_rect);
    cairo_fill(cr);
  }

  cairo_destroy(cr);
}
